/*
** distsbr.c -- routines to do additional "dist-style" processing
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <fcntl.h>
#include <h/utils.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>

static int  hdrfd = NOTOK;
static int  txtfd = NOTOK;

/*
** static prototypes
*/
static int ready_msg(char *);

int
distout(char *drft, char *msgnam, char *backup)
{
	int state;
	unsigned char *dp;
	char *resent;
	char name[NAMESZ], buffer[BUFSIZ];
	FILE *ifp, *ofp;

	strcpy(backup, m_mktemp(toabsdir(invo_name), NULL, NULL));
	if (rename(drft, backup) == NOTOK) {
		advise(backup, "unable to rename %s to",drft);
		return NOTOK;
	}
	if (!(ifp = fopen(backup, "r"))) {
		advise(backup, "unable to read");
		return NOTOK;
	}

	if (!(ofp = fopen(drft, "w"))) {
		advise(drft, "unable to create temporary file");
		return NOTOK;
	}
	chmod(drft, m_gmprot());

	if (ready_msg(msgnam) != OK) {
		return NOTOK;
	}
	lseek(hdrfd, (off_t) 0, SEEK_SET); /* msgnam not accurate */
	cpydata(hdrfd, fileno(ofp), msgnam, drft);

	state = FLD;
	resent = NULL;
	while (1) {
		switch (state = m_getfld(state, name, buffer, sizeof buffer,
				ifp)) {
		case FLD:
		case FLDPLUS:
		case FLDEOF:
			if (!uprf(name, "resent")) {
				advise(NULL, "Please re-edit draft to remove the ``%s'' header.", name);
				goto leave_bad;
			}
			if (state == FLD) {
				resent = add(":", add(name, resent));
			}
			resent = add(buffer, resent);
			fprintf(ofp, "%s: %s", name, buffer);
			while (state == FLDPLUS) {
				state = m_getfld(state, name, buffer,
						sizeof buffer, ifp);
				resent = add(buffer, resent);
				fputs(buffer, ofp);
			}
			if (state == FLDEOF) {
				goto process;
			}
			break;

		case BODY:
		case BODYEOF:
			for (dp = buffer; *dp; dp++) {
				if (!isspace(*dp)) {
					advise(NULL, "Please re-edit draft to consist of headers only.");
					goto leave_bad;
				}
			}

		case FILEEOF:
			goto process;

		case LENERR:
		case FMTERR:
			advise(NULL, "Please re-edit draft and fix that header.");
leave_bad: ;
			fclose(ifp);
			fclose(ofp);
			unlink(drft);
			if (rename(backup, drft) == NOTOK) {
				advise(drft, "unable to rename %s to", backup);
			}
			return NOTOK;

		default:
			advise(NULL, "getfld() returned %d", state);
			return NOTOK;
		}
	}

process: ;
	fclose(ifp);
	fflush(ofp);

	if (!resent) {
		advise(NULL, "Please re-edit draft to include a ``Resent-To:'' header.");
		fclose(ofp);
		unlink(drft);
		if (rename(backup, drft) == NOTOK) {
			advise(drft, "unable to rename %s to", backup);
		}
		return NOTOK;
	}
	free(resent);

	if (txtfd != NOTOK) {
		lseek(txtfd, (off_t) 0, SEEK_SET);  /* msgnam not accurate */
		cpydata(txtfd, fileno(ofp), msgnam, drft);
	}

	fclose(ofp);

	return OK;
}


static int
ready_msg(char *msgnam)
{
	int state, out;
	char name[NAMESZ], buffer[BUFSIZ], tmpfil[BUFSIZ];
	FILE *ifp, *ofp;
	char *cp = NULL;

	if (hdrfd != NOTOK) {
		close(hdrfd);
		hdrfd = NOTOK;
	}
	if (txtfd != NOTOK) {
		close(txtfd);
		txtfd = NOTOK;
	}
	if (!(ifp = fopen(msgnam, "r"))) {
		advise(msgnam, "unable to open message");
		return NOTOK;
	}

	cp = m_mktemp2(NULL, "dist", &hdrfd, NULL);
	if (!cp) {
		advise("distsbr", "unable to create temporary file");
		return NOTOK;
	}
	fchmod(hdrfd, 0600);
	strncpy(tmpfil, cp, sizeof(tmpfil));
	if ((out = dup(hdrfd)) == NOTOK || !(ofp = fdopen(out, "w"))) {
		advise(NULL, "no file descriptors -- you lose big");
		return NOTOK;
	}
	unlink(tmpfil);

	state = FLD;
	while (1) {
		state = m_getfld(state, name, buffer, sizeof buffer, ifp);
		switch (state) {
		case FLD:
		case FLDPLUS:
		case FLDEOF:
			if (uprf(name, "resent")) {
				fprintf(ofp, "Prev-");
			}
			fprintf(ofp, "%s: %s", name, buffer);
			while (state == FLDPLUS) {
				state = m_getfld(state, name, buffer,
						sizeof buffer, ifp);
				fputs(buffer, ofp);
			}
			if (state == FLDEOF) {
				goto process;
			}
			break;

		case BODY:
		case BODYEOF:
			fclose(ofp);

			cp = m_mktemp2(NULL, "dist", &txtfd, NULL);
			if (!cp) {
				advise("distsbr", "unable to create temp file");
				return NOTOK;
			}
			fchmod(txtfd, 0600);
			strncpy(tmpfil, cp, sizeof(tmpfil));
			if ((out = dup(txtfd)) == NOTOK ||
					!(ofp = fdopen(out, "w"))) {
				advise(NULL, "no file descriptors -- you lose");
				return NOTOK;
			}
			unlink(tmpfil);
			fprintf(ofp, "\n%s", buffer);
			while (state == BODY) {
				state = m_getfld(state, name, buffer,
						sizeof buffer, ifp);
				fputs(buffer, ofp);
			}
		case FILEEOF:
			goto process;

		case LENERR:
		case FMTERR:
			advise(NULL, "format error in message %s", msgnam);
			return NOTOK;

		default:
			advise(NULL, "getfld() returned %d", state);
			return NOTOK;
		}
	}
process: ;
	fclose(ifp);
	fclose(ofp);
	return OK;
}
